@@ -1,14 +1,22 @@
-from hcloud.core.client import ClientEntityBase, BoundModelBase, GetEntityByNameMixin
+from __future__ import annotations
 
-from hcloud.datacenters.domain import Datacenter, DatacenterServerTypes
-from hcloud.locations.client import BoundLocation
-from hcloud.server_types.client import BoundServerType
+from typing import TYPE_CHECKING, Any, NamedTuple
+
+from ..core import BoundModelBase, ClientEntityBase, Meta
+from ..locations import BoundLocation
+from ..server_types import BoundServerType
+from .domain import Datacenter, DatacenterServerTypes
+
+if TYPE_CHECKING:
+    from .._client import Client
 
 
-class BoundDatacenter(BoundModelBase):
+class BoundDatacenter(BoundModelBase, Datacenter):
+    _client: DatacentersClient
+
     model = Datacenter
 
-    def __init__(self, client, data):
+    def __init__(self, client: DatacentersClient, data: dict):
         location = data.get("location")
         if location is not None:
             data["location"] = BoundLocation(client._client.locations, location)
@@ -39,31 +47,32 @@ class BoundDatacenter(BoundModelBase):
                 available_for_migration=available_for_migration,
             )
 
-        super(BoundDatacenter, self).__init__(client, data)
+        super().__init__(client, data)
 
 
-class DatacentersClient(ClientEntityBase, GetEntityByNameMixin):
-    results_list_attribute_name = "datacenters"
+class DatacentersPageResult(NamedTuple):
+    datacenters: list[BoundDatacenter]
+    meta: Meta | None
 
-    def get_by_id(self, id):
-        # type: (int) -> BoundDatacenter
+
+class DatacentersClient(ClientEntityBase):
+    _client: Client
+
+    def get_by_id(self, id: int) -> BoundDatacenter:
         """Get a specific datacenter by its ID.
 
         :param id: int
         :return: :class:`BoundDatacenter <hcloud.datacenters.client.BoundDatacenter>`
         """
-        response = self._client.request(
-            url="/datacenters/{datacenter_id}".format(datacenter_id=id), method="GET"
-        )
+        response = self._client.request(url=f"/datacenters/{id}", method="GET")
         return BoundDatacenter(self, response["datacenter"])
 
     def get_list(
         self,
-        name=None,  # type: Optional[str]
-        page=None,  # type: Optional[int]
-        per_page=None,  # type: Optional[int]
-    ):
-        # type: (...) -> PageResults[List[BoundDatacenter], Meta]
+        name: str | None = None,
+        page: int | None = None,
+        per_page: int | None = None,
+    ) -> DatacentersPageResult:
         """Get a list of datacenters
 
         :param name: str (optional)
@@ -74,7 +83,7 @@ class DatacentersClient(ClientEntityBase, GetEntityByNameMixin):
                Specifies how many results are returned by page
         :return: (List[:class:`BoundDatacenter <hcloud.datacenters.client.BoundDatacenter>`], :class:`Meta <hcloud.core.domain.Meta>`)
         """
-        params = {}
+        params: dict[str, Any] = {}
         if name is not None:
             params["name"] = name
 
@@ -91,24 +100,22 @@ class DatacentersClient(ClientEntityBase, GetEntityByNameMixin):
             for datacenter_data in response["datacenters"]
         ]
 
-        return self._add_meta_to_result(datacenters, response)
+        return DatacentersPageResult(datacenters, Meta.parse_meta(response))
 
-    def get_all(self, name=None):
-        # type: (Optional[str]) -> List[BoundDatacenter]
+    def get_all(self, name: str | None = None) -> list[BoundDatacenter]:
         """Get all datacenters
 
         :param name: str (optional)
                Can be used to filter datacenters by their name.
         :return: List[:class:`BoundDatacenter <hcloud.datacenters.client.BoundDatacenter>`]
         """
-        return super(DatacentersClient, self).get_all(name=name)
+        return self._iter_pages(self.get_list, name=name)
 
-    def get_by_name(self, name):
-        # type: (str) -> BoundDatacenter
+    def get_by_name(self, name: str) -> BoundDatacenter | None:
         """Get datacenter by name
 
         :param name: str
                Used to get datacenter by name.
         :return: :class:`BoundDatacenter <hcloud.datacenters.client.BoundDatacenter>`
         """
-        return super(DatacentersClient, self).get_by_name(name)
+        return self._get_first_by(name=name)
