@@ -1,13 +1,24 @@
-from ..actions.client import BoundAction
-from ..core.client import BoundModelBase, ClientEntityBase, GetEntityByNameMixin
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Any, NamedTuple
+
+from ..actions import BoundAction, ResourceActionsClient
+from ..core import BoundModelBase, ClientEntityBase, Meta
 from .domain import CreatePrimaryIPResponse, PrimaryIP
 
+if TYPE_CHECKING:
+    from .._client import Client
+    from ..datacenters import BoundDatacenter, Datacenter
+
+
+class BoundPrimaryIP(BoundModelBase, PrimaryIP):
+    _client: PrimaryIPsClient
 
-class BoundPrimaryIP(BoundModelBase):
     model = PrimaryIP
 
-    def __init__(self, client, data, complete=True):
-        from ..datacenters.client import BoundDatacenter
+    def __init__(self, client: PrimaryIPsClient, data: dict, complete: bool = True):
+        # pylint: disable=import-outside-toplevel
+        from ..datacenters import BoundDatacenter
 
         datacenter = data.get("datacenter", {})
         if datacenter:
@@ -15,8 +26,12 @@ class BoundPrimaryIP(BoundModelBase):
 
         super().__init__(client, data, complete)
 
-    def update(self, auto_delete=None, labels=None, name=None):
-        # type: (Optional[bool], Optional[Dict[str, str]], Optional[str]) -> BoundPrimaryIP
+    def update(
+        self,
+        auto_delete: bool | None = None,
+        labels: dict[str, str] | None = None,
+        name: str | None = None,
+    ) -> BoundPrimaryIP:
         """Updates the description or labels of a Primary IP.
 
         :param auto_delete: bool (optional)
@@ -31,16 +46,14 @@ class BoundPrimaryIP(BoundModelBase):
             self, auto_delete=auto_delete, labels=labels, name=name
         )
 
-    def delete(self):
-        # type: () -> bool
+    def delete(self) -> bool:
         """Deletes a Primary IP. If it is currently assigned to a server it will automatically get unassigned.
 
         :return: boolean
         """
         return self._client.delete(self)
 
-    def change_protection(self, delete=None):
-        # type: (Optional[bool]) -> BoundAction
+    def change_protection(self, delete: bool | None = None) -> BoundAction:
         """Changes the protection configuration of the Primary IP.
 
         :param delete: boolean
@@ -49,8 +62,7 @@ class BoundPrimaryIP(BoundModelBase):
         """
         return self._client.change_protection(self, delete)
 
-    def assign(self, assignee_id, assignee_type):
-        # type: (int,str) -> BoundAction
+    def assign(self, assignee_id: int, assignee_type: str) -> BoundAction:
         """Assigns a Primary IP to a assignee.
 
         :param assignee_id: int`
@@ -61,16 +73,14 @@ class BoundPrimaryIP(BoundModelBase):
         """
         return self._client.assign(self, assignee_id, assignee_type)
 
-    def unassign(self):
-        # type: () -> BoundAction
+    def unassign(self) -> BoundAction:
         """Unassigns a Primary IP, resulting in it being unreachable. You may assign it to a server again at a later time.
 
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         return self._client.unassign(self)
 
-    def change_dns_ptr(self, ip, dns_ptr):
-        # type: (str, str) -> BoundAction
+    def change_dns_ptr(self, ip: str, dns_ptr: str) -> BoundAction:
         """Changes the hostname that will appear when getting the hostname belonging to this Primary IP.
 
         :param ip: str
@@ -82,11 +92,25 @@ class BoundPrimaryIP(BoundModelBase):
         return self._client.change_dns_ptr(self, ip, dns_ptr)
 
 
-class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
-    results_list_attribute_name = "primary_ips"
+class PrimaryIPsPageResult(NamedTuple):
+    primary_ips: list[BoundPrimaryIP]
+    meta: Meta | None
 
-    def get_by_id(self, id):
-        # type: (int) -> BoundPrimaryIP
+
+class PrimaryIPsClient(ClientEntityBase):
+    _client: Client
+
+    actions: ResourceActionsClient
+    """Primary IPs scoped actions client
+
+    :type: :class:`ResourceActionsClient <hcloud.actions.client.ResourceActionsClient>`
+    """
+
+    def __init__(self, client: Client):
+        super().__init__(client)
+        self.actions = ResourceActionsClient(client, "/primary_ips")
+
+    def get_by_id(self, id: int) -> BoundPrimaryIP:
         """Returns a specific Primary IP object.
 
         :param id: int
@@ -97,13 +121,12 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
 
     def get_list(
         self,
-        label_selector=None,  # type: Optional[str]
-        page=None,  # type: Optional[int]
-        per_page=None,  # type: Optional[int]
-        name=None,  # type: Optional[str]
-        ip=None,  # type: Optional[ip]
-    ):
-        # type: (...) -> PageResults[List[BoundPrimaryIP]]
+        label_selector: str | None = None,
+        page: int | None = None,
+        per_page: int | None = None,
+        name: str | None = None,
+        ip: str | None = None,
+    ) -> PrimaryIPsPageResult:
         """Get a list of primary ips from this account
 
         :param label_selector: str (optional)
@@ -118,7 +141,7 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
                Can be used to filter resources by their ip. The response will only contain the resources matching the specified ip.
         :return: (List[:class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>`], :class:`Meta <hcloud.core.domain.Meta>`)
         """
-        params = {}
+        params: dict[str, Any] = {}
 
         if label_selector is not None:
             params["label_selector"] = label_selector
@@ -137,10 +160,13 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
             for primary_ip_data in response["primary_ips"]
         ]
 
-        return self._add_meta_to_result(primary_ips, response)
+        return PrimaryIPsPageResult(primary_ips, Meta.parse_meta(response))
 
-    def get_all(self, label_selector=None, name=None):
-        # type: (Optional[str], Optional[str]) -> List[BoundPrimaryIP]
+    def get_all(
+        self,
+        label_selector: str | None = None,
+        name: str | None = None,
+    ) -> list[BoundPrimaryIP]:
         """Get all primary ips from this account
 
         :param label_selector: str (optional)
@@ -149,29 +175,28 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
                Can be used to filter networks by their name.
         :return: List[:class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>`]
         """
-        return super().get_all(label_selector=label_selector, name=name)
+        return self._iter_pages(self.get_list, label_selector=label_selector, name=name)
 
-    def get_by_name(self, name):
-        # type: (str) -> BoundPrimaryIP
+    def get_by_name(self, name: str) -> BoundPrimaryIP | None:
         """Get Primary IP by name
 
         :param name: str
                Used to get Primary IP by name.
         :return: :class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>`
         """
-        return super().get_by_name(name)
+        return self._get_first_by(name=name)
 
     def create(
         self,
-        type,  # type: str
-        datacenter,  # type: Datacenter
-        name,  # type: str
-        assignee_type="server",  # type: Optional[str]
-        assignee_id=None,  # type: Optional[int]
-        auto_delete=False,  # type: Optional[bool]
-        labels=None,  # type: Optional[dict]
-    ):
-        # type: (...) -> CreatePrimaryIPResponse
+        type: str,
+        # TODO: Make the datacenter argument optional
+        datacenter: Datacenter | BoundDatacenter | None,
+        name: str,
+        assignee_type: str | None = "server",
+        assignee_id: int | None = None,
+        auto_delete: bool | None = False,
+        labels: dict | None = None,
+    ) -> CreatePrimaryIPResponse:
         """Creates a new Primary IP assigned to a server.
 
         :param type: str
@@ -186,14 +211,15 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
         :return: :class:`CreatePrimaryIPResponse <hcloud.primary_ips.domain.CreatePrimaryIPResponse>`
         """
 
-        data = {
+        data: dict[str, Any] = {
             "type": type,
             "assignee_type": assignee_type,
             "auto_delete": auto_delete,
-            "datacenter": datacenter.id_or_name,
             "name": name,
         }
-        if assignee_id:
+        if datacenter is not None:
+            data["datacenter"] = datacenter.id_or_name
+        if assignee_id is not None:
             data["assignee_id"] = assignee_id
         if labels is not None:
             data["labels"] = labels
@@ -209,8 +235,13 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
         )
         return result
 
-    def update(self, primary_ip, auto_delete=None, labels=None, name=None):
-        # type: (PrimaryIP,  Optional[bool], Optional[Dict[str, str]], Optional[str]) -> BoundPrimaryIP
+    def update(
+        self,
+        primary_ip: PrimaryIP | BoundPrimaryIP,
+        auto_delete: bool | None = None,
+        labels: dict[str, str] | None = None,
+        name: str | None = None,
+    ) -> BoundPrimaryIP:
         """Updates the name, auto_delete or labels of a Primary IP.
 
         :param primary_ip: :class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>` or  :class:`PrimaryIP <hcloud.primary_ips.domain.PrimaryIP>`
@@ -222,7 +253,7 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
                New name to set
         :return: :class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>`
         """
-        data = {}
+        data: dict[str, Any] = {}
         if auto_delete is not None:
             data["auto_delete"] = auto_delete
         if labels is not None:
@@ -237,8 +268,7 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
         )
         return BoundPrimaryIP(self, response["primary_ip"])
 
-    def delete(self, primary_ip):
-        # type: (PrimaryIP) -> bool
+    def delete(self, primary_ip: PrimaryIP | BoundPrimaryIP) -> bool:
         """Deletes a Primary IP. If it is currently assigned to an assignee it will automatically get unassigned.
 
         :param primary_ip: :class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>` or  :class:`PrimaryIP <hcloud.primary_ips.domain.PrimaryIP>`
@@ -251,8 +281,11 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
         # Return always true, because the API does not return an action for it. When an error occurs a HcloudAPIException will be raised
         return True
 
-    def change_protection(self, primary_ip, delete=None):
-        # type: (PrimaryIP, Optional[bool]) -> BoundAction
+    def change_protection(
+        self,
+        primary_ip: PrimaryIP | BoundPrimaryIP,
+        delete: bool | None = None,
+    ) -> BoundAction:
         """Changes the protection configuration of the Primary IP.
 
         :param primary_ip: :class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>` or  :class:`PrimaryIP <hcloud.primary_ips.domain.PrimaryIP>`
@@ -260,21 +293,23 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
                If true, prevents the Primary IP from being deleted
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        data = {}
+        data: dict[str, Any] = {}
         if delete is not None:
             data.update({"delete": delete})
 
         response = self._client.request(
-            url="/primary_ips/{primary_ip_id}/actions/change_protection".format(
-                primary_ip_id=primary_ip.id
-            ),
+            url=f"/primary_ips/{primary_ip.id}/actions/change_protection",
             method="POST",
             json=data,
         )
         return BoundAction(self._client.actions, response["action"])
 
-    def assign(self, primary_ip, assignee_id, assignee_type="server"):
-        # type: (PrimaryIP, int, str) -> BoundAction
+    def assign(
+        self,
+        primary_ip: PrimaryIP | BoundPrimaryIP,
+        assignee_id: int,
+        assignee_type: str = "server",
+    ) -> BoundAction:
         """Assigns a Primary IP to a assignee_id.
 
         :param primary_ip: :class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>` or  :class:`PrimaryIP <hcloud.primary_ips.domain.PrimaryIP>`
@@ -285,31 +320,30 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url="/primary_ips/{primary_ip_id}/actions/assign".format(
-                primary_ip_id=primary_ip.id
-            ),
+            url=f"/primary_ips/{primary_ip.id}/actions/assign",
             method="POST",
             json={"assignee_id": assignee_id, "assignee_type": assignee_type},
         )
         return BoundAction(self._client.actions, response["action"])
 
-    def unassign(self, primary_ip):
-        # type: (PrimaryIP) -> BoundAction
+    def unassign(self, primary_ip: PrimaryIP | BoundPrimaryIP) -> BoundAction:
         """Unassigns a Primary IP, resulting in it being unreachable. You may assign it to a server again at a later time.
 
         :param primary_ip: :class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>` or  :class:`PrimaryIP <hcloud.primary_ips.domain.PrimaryIP>`
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url="/primary_ips/{primary_ip_id}/actions/unassign".format(
-                primary_ip_id=primary_ip.id
-            ),
+            url=f"/primary_ips/{primary_ip.id}/actions/unassign",
             method="POST",
         )
         return BoundAction(self._client.actions, response["action"])
 
-    def change_dns_ptr(self, primary_ip, ip, dns_ptr):
-        # type: (PrimaryIP, str, str) -> BoundAction
+    def change_dns_ptr(
+        self,
+        primary_ip: PrimaryIP | BoundPrimaryIP,
+        ip: str,
+        dns_ptr: str,
+    ) -> BoundAction:
         """Changes the dns ptr that will appear when getting the dns ptr belonging to this Primary IP.
 
         :param primary_ip: :class:`BoundPrimaryIP <hcloud.primary_ips.client.BoundPrimaryIP>` or  :class:`PrimaryIP <hcloud.primary_ips.domain.PrimaryIP>`
@@ -320,9 +354,7 @@ class PrimaryIPsClient(ClientEntityBase, GetEntityByNameMixin):
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url="/primary_ips/{primary_ip_id}/actions/change_dns_ptr".format(
-                primary_ip_id=primary_ip.id
-            ),
+            url=f"/primary_ips/{primary_ip.id}/actions/change_dns_ptr",
             method="POST",
             json={"ip": ip, "dns_ptr": dns_ptr},
         )
