@@ -1,16 +1,12 @@
+from __future__ import annotations
+
 import pytest
 from dateutil.parser import isoparse
 
-from hcloud.core.domain import (
-    BaseDomain,
-    DomainIdentityMixin,
-    Meta,
-    Pagination,
-    add_meta_to_result,
-)
+from hcloud.core import BaseDomain, DomainIdentityMixin, Meta, Pagination
 
 
-class TestMeta(object):
+class TestMeta:
     @pytest.mark.parametrize("json_content", [None, "", {}])
     def test_parse_meta_empty_json(self, json_content):
         result = Meta.parse_meta(json_content)
@@ -44,27 +40,6 @@ class TestMeta(object):
         assert result.pagination.last_page == 10
         assert result.pagination.total_entries == 100
 
-    def test_add_meta_to_result(self):
-        json_content = {
-            "meta": {
-                "pagination": {
-                    "page": 2,
-                    "per_page": 10,
-                    "previous_page": 1,
-                    "next_page": 3,
-                    "last_page": 10,
-                    "total_entries": 100,
-                }
-            }
-        }
-        result = add_meta_to_result([1, 2, 3], json_content, "id_list")
-        assert result.id_list == [1, 2, 3]
-        assert result.meta.pagination.page == 2
-        assert result.meta.pagination.per_page == 10
-        assert result.meta.pagination.next_page == 3
-        assert result.meta.pagination.last_page == 10
-        assert result.meta.pagination.total_entries == 100
-
 
 class SomeDomain(BaseDomain, DomainIdentityMixin):
     __slots__ = ("id", "name")
@@ -74,7 +49,7 @@ class SomeDomain(BaseDomain, DomainIdentityMixin):
         self.name = name
 
 
-class TestDomainIdentityMixin(object):
+class TestDomainIdentityMixin:
     @pytest.mark.parametrize(
         "domain,expected_result",
         [
@@ -104,7 +79,16 @@ class ActionDomain(BaseDomain, DomainIdentityMixin):
         self.started = isoparse(started) if started else None
 
 
-class TestBaseDomain(object):
+class SomeOtherDomain(BaseDomain):
+    __slots__ = ("id", "name", "child")
+
+    def __init__(self, id=None, name=None, child=None):
+        self.id = id
+        self.name = name
+        self.child = child
+
+
+class TestBaseDomain:
     @pytest.mark.parametrize(
         "data_dict,expected_result",
         [
@@ -134,3 +118,23 @@ class TestBaseDomain(object):
         model = ActionDomain.from_dict(data_dict)
         for k, v in expected_result.items():
             assert getattr(model, k) == v
+
+    @pytest.mark.parametrize(
+        "data,expected",
+        [
+            (
+                SomeOtherDomain(id=1, name="name1"),
+                "SomeOtherDomain(id=1, name='name1', child=None)",
+            ),
+            (
+                SomeOtherDomain(
+                    id=2,
+                    name="name2",
+                    child=SomeOtherDomain(id=3, name="name3"),
+                ),
+                "SomeOtherDomain(id=2, name='name2', child=SomeOtherDomain(id=3, name='name3', child=None))",
+            ),
+        ],
+    )
+    def test_repr_ok(self, data, expected):
+        assert data.__repr__() == expected
