@@ -1,18 +1,22 @@
-import pytest
+from __future__ import annotations
+
 from unittest import mock
 
-from hcloud.firewalls.client import FirewallsClient, BoundFirewall
-from hcloud.actions.client import BoundAction
-from hcloud.firewalls.domain import (
+import pytest
+
+from hcloud.actions import BoundAction
+from hcloud.firewalls import (
+    BoundFirewall,
     Firewall,
-    FirewallRule,
     FirewallResource,
     FirewallResourceLabelSelector,
+    FirewallRule,
+    FirewallsClient,
 )
-from hcloud.servers.domain import Server
+from hcloud.servers import Server
 
 
-class TestBoundFirewall(object):
+class TestBoundFirewall:
     @pytest.fixture()
     def bound_firewall(self, hetzner_client):
         return BoundFirewall(client=hetzner_client.firewalls, data=dict(id=1))
@@ -84,6 +88,7 @@ class TestBoundFirewall(object):
 
         assert len(actions) == 1
         assert isinstance(actions[0], BoundAction)
+        assert actions[0]._client == hetzner_client.actions
         assert actions[0].id == 13
         assert actions[0].command == "set_firewall_rules"
 
@@ -102,6 +107,7 @@ class TestBoundFirewall(object):
 
         assert len(actions) == 1
         assert isinstance(actions[0], BoundAction)
+        assert actions[0]._client == hetzner_client.actions
         assert actions[0].id == 13
         assert actions[0].command == "set_firewall_rules"
 
@@ -188,7 +194,7 @@ class TestBoundFirewall(object):
         assert actions[0].progress == 100
 
 
-class TestFirewallsClient(object):
+class TestFirewallsClient:
     @pytest.fixture()
     def firewalls_client(self):
         return FirewallsClient(client=mock.MagicMock())
@@ -456,3 +462,53 @@ class TestFirewallsClient(object):
 
         assert actions[0].id == 13
         assert actions[0].progress == 100
+
+    def test_actions_get_by_id(self, firewalls_client, response_get_actions):
+        firewalls_client._client.request.return_value = {
+            "action": response_get_actions["actions"][0]
+        }
+        action = firewalls_client.actions.get_by_id(13)
+
+        firewalls_client._client.request.assert_called_with(
+            url="/firewalls/actions/13", method="GET"
+        )
+
+        assert isinstance(action, BoundAction)
+        assert action._client == firewalls_client._client.actions
+        assert action.id == 13
+        assert action.command == "set_firewall_rules"
+
+    def test_actions_get_list(self, firewalls_client, response_get_actions):
+        firewalls_client._client.request.return_value = response_get_actions
+        result = firewalls_client.actions.get_list()
+
+        firewalls_client._client.request.assert_called_with(
+            url="/firewalls/actions",
+            method="GET",
+            params={},
+        )
+
+        actions = result.actions
+        assert result.meta is None
+
+        assert len(actions) == 1
+        assert isinstance(actions[0], BoundAction)
+        assert actions[0]._client == firewalls_client._client.actions
+        assert actions[0].id == 13
+        assert actions[0].command == "set_firewall_rules"
+
+    def test_actions_get_all(self, firewalls_client, response_get_actions):
+        firewalls_client._client.request.return_value = response_get_actions
+        actions = firewalls_client.actions.get_all()
+
+        firewalls_client._client.request.assert_called_with(
+            url="/firewalls/actions",
+            method="GET",
+            params={"page": 1, "per_page": 50},
+        )
+
+        assert len(actions) == 1
+        assert isinstance(actions[0], BoundAction)
+        assert actions[0]._client == firewalls_client._client.actions
+        assert actions[0].id == 13
+        assert actions[0].command == "set_firewall_rules"
