@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 import datetime
 from datetime import timezone
 
-from hcloud.firewalls.domain import Firewall
+from hcloud.firewalls import Firewall
 
 
 class TestFirewall:
